/*
 * process, Distribution repositories and basic setup for Emmanuel Bigeon projects
 * Copyright (C) 2014-2018 E. Bigeon
 * mailto:emmanuel@bigeon.fr
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-process:fr.bigeon.gclc.process.CommandFork.java
 * Created on: Nov 13, 2017
 */
package fr.bigeon.gclc.process;

import java.util.List;

import fr.bigeon.gclc.command.CommandParameters;
import fr.bigeon.gclc.command.ParametrizedCommand;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.CommandRunExceptionType;
import fr.bigeon.gclc.exception.InvalidParameterException;
import fr.bigeon.gclc.manager.ConsoleInput;
import fr.bigeon.gclc.manager.ConsoleOutput;

/** A command that is launched inside an internal terminal.
 * <p>
 * Several things are to be considered before adding this command to an
 * application:
 * <ul>
 * <li>The commands will be able to run in parallel.
 * <li>The managing of the commands will be handled through piped systems and
 * buffered in so switching to a forked command will actually print again all
 * the command history to the output console.
 * </ul>
 *
 * @author Emmanuel Bigeon */
public class CommandForeground extends ParametrizedCommand {

    private final TaskPool pool;

    /** Add the fork command.
     *
     * @param name the command name
     * @param pool The pool to get joinable tasks from */
    public CommandForeground(final String name, TaskPool pool) {
        super(name, false);
        this.pool = pool;
        addParameters();
    }

    /**
     *
     */
    private void addParameters() {
        try {
            addStringParameter("pid", false);
            addStringParameter("delai", false);
        } catch (final InvalidParameterException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ParametrizedCommand#doExecute(fr.bigeon.gclc.
     * manager.ConsoleOutput, fr.bigeon.gclc.manager.ConsoleInput,
     * fr.bigeon.gclc.command.CommandParameters) */
    @Override
    protected void doExecute(final ConsoleOutput out, final ConsoleInput in,
            final CommandParameters parameters) throws CommandRunException {
        String string = parameters.get("pid");
        final List<String> additionals = parameters.getAdditionals();
        if (string == null && !additionals.isEmpty()) {
            string = additionals.get(0);
        }
        if (string == null) {
            throw new CommandRunException(CommandRunExceptionType.USAGE,
                    "Missing process id", this);
        }
        // Join the command.
        final Task cmd = pool.get(string);
        if (!(cmd instanceof ForkTask)) {
            throw new CommandRunException("No such forked process", this);
        }
        long delai = 0;
        final String delaiOpt = parameters.get("delai");
        if (delaiOpt != null) {
            delai = Long.parseLong(delaiOpt) * 1000;
        }
        if (delai < 0) {
            throw new CommandRunException("Join delai cannot be negative", this);
        }
        ((ForkTask) cmd).join(out, in, delai);
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#tip() */
    @Override
    public String tip() {
        return "Join the execution of a command, for a given amount of time";
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usageDetail() */
    @Override
    protected String usageDetail() {
        return "";
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usagePattern() */
    @Override
    protected String usagePattern() {
        return super.usagePattern() + " [(-pid) <id>] ([-delai <delai>])";
    }
}
