/*
 * process, Distribution repositories and basic setup for Emmanuel Bigeon projects
 * Copyright (C) 2014-2018 E. Bigeon
 * mailto:emmanuel@bigeon.fr
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-process:fr.bigeon.gclc.process.CommandFork.java
 * Created on: Nov 13, 2017
 */
package fr.bigeon.gclc.process;

import java.util.Arrays;

import fr.bigeon.gclc.command.Command;
import fr.bigeon.gclc.command.ICommand;
import fr.bigeon.gclc.command.ICommandProvider;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.manager.ConsoleInput;
import fr.bigeon.gclc.manager.ConsoleOutput;

/** A command that is launched inside an internal terminal.
 * <p>
 * Several things are to be considered before adding this command to an
 * application:
 * <ul>
 * <li>The commands will be able to run in parallel.
 * <li>The managing of the commands will be handled through piped systems and
 * buffered in so switching to a forked command will actually print again all
 * the command history to the output console.
 * </ul>
 *
 * @author Emmanuel Bigeon */
public class CommandFork extends Command {

    private final TaskPool         pool;
    private final ICommandProvider provider;
    private final int              lines;

    /** Add the fork command.
     *
     * @param name the command name
     * @param provider the allowed command collection */
    public CommandFork(final String name, ICommandProvider provider, TaskPool pool) {
        this(name, provider, pool, -1);
    }

    /** Add the fork command.
     *
     * @param name the command name
     * @param provider the allowed command collection */
    public CommandFork(final String name, ICommandProvider provider, TaskPool pool,
                       int lines) {
        super(name);
        this.provider = provider;
        this.pool = pool;
        this.lines = lines;
    }

    @Override
    public void execute(final ConsoleOutput out, final ConsoleInput in, String... args)
            throws CommandRunException {
        if (args.length < 1) {
            throw new CommandRunException("No command to fork", this);
        }
        final ICommand cmd = provider.get(args[0]);
        final String[] inner = Arrays.copyOfRange(args, 1, args.length);
        final ForkTask task = new ForkCommandTask(cmd, inner, lines);
        final Thread th = new Thread(task);
        pool.add(task);
        th.start();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#tip() */
    @Override
    public String tip() {
        return "Command background launch";
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usageDetail() */
    @Override
    protected String usageDetail() {
        return "";
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usagePattern() */
    @Override
    protected String usagePattern() {
        return super.usagePattern() + " <command>";
    }
}
